#include "framebuffer.h"
#include "v3.h"
#include <iostream>
#include "scene.h"
#include <math.h>

FrameBuffer::FrameBuffer(int u0, int v0, 
  int _w, int _h) : Fl_Gl_Window(u0, v0, _w, _h, 0) {

  w = _w;
  h = _h;
  pix = new unsigned int[w*h];
  Set(0xFFFFFFFF);

}


void FrameBuffer::draw() {

  glDrawPixels(w, h, GL_RGBA, GL_UNSIGNED_BYTE, pix);

}

void FrameBuffer::Set(unsigned int color) {

  for (int i = 0; i < w*h; i++)
    pix[i] = color;

}

void FrameBuffer::Set(int u, int v, unsigned int color) {

  pix[w*(h-1-v)+u] = color;

}

unsigned int FrameBuffer::Get(int u, int v) {

  return pix[w*(h-1-v)+u];

}

void FrameBuffer::SetAsChecker(int cs, unsigned int cols[2]) {

  for (int v = 0; v < h; v++) {
    for (int u = 0; u < w; u++) {
      int cu = u/cs;
      int cv = v/cs;
      int i = (cu+cv)%2;
      Set(u, v, cols[i]);
    }
  }

}

void FrameBuffer::DrawSegment(V3 P0, V3 P1, V3 c0, V3 c1) {

  int pixN;
  float du = fabsf(P0[0]-P1[0]);
  float dv = fabsf(P0[1]-P1[1]);
  if (du < dv) {
    pixN = 1+(int)dv;
  }
  else {
    pixN = 1+(int)du;
  }

  V3 currP = P0;
  V3 dP = (P1-P0)/(float)pixN;
  V3 currC = c0;
  V3 dC = (c1-c0)/(float)pixN;
  for (int si = 0; si < pixN+1; si++) {
    SetGuarded(currP, currC.GetColor());
    currP = currP + dP;
    currC = currC + dC;
  }

}

void FrameBuffer::Set(V3 P, unsigned int color) {

  int u = (int) P[0];
  int v = (int) P[1];
  Set(u, v, color);

}

void FrameBuffer::SetGuarded(V3 P, unsigned int color) {

  int u = (int) P[0];
  int v = (int) P[1];
  if (u < 0 || u > w-1 || v < 0 || v > h-1)
    return;
  Set(u, v, color);

}


void FrameBuffer::Brighten(float scf) {

  for (int v = 0; v < h; v++) {
    for (int u = 0; u < w; u++) {
      V3 col;
      col.SetFromColor(Get(u, v));
      col = col * scf;
      Set(u, v, col.GetColor());
    }
  }

}

void FrameBuffer::FindEdges(FrameBuffer *edgeMap) {

  // traversing the entire image
  for (int v = 1; v < h-1; v++) {
    for (int u = 1; u < w-1; u++) {
      // compute convolution at current pixel
      V3 conv(0.0f, 0.0f, 0.0f);
      conv = Getv(u, v)*4.0f - Getv(u-1, v)
        - Getv(u+1, v) - Getv(u, v-1) - Getv(u, v+1);
      // take absolute value of each channel
      // write in edge map
    }
  }

}

V3 FrameBuffer::Getv(int u, int v) {


  cerr << "ERROR: not yet implemented" << endl;
  return V3(0.0f, 0.0f, 0.0f);

}


int FrameBuffer::handle(int event)  {  

  switch(event) 
  {   
  case FL_KEYBOARD: {
    KeyboardHandle();
    return 0;
  }
  default:
    break;
  }
  return 0;

}


void FrameBuffer::KeyboardHandle() {

  float tstep = 3.0f;
  float rstep = 3.0f;
  int key = Fl::event_key();
  switch (key) {
    case 'a': {
      scene->ppc->TranslateLeft(tstep);
      scene->Render();
      break;
    }
    case 'd': {
      scene->ppc->TranslateRight(tstep);
      scene->Render();
      break;
    }
    case 'w': {
      scene->ppc->TranslateUp(tstep);
      scene->Render();
      break;
    }
    case 's': {
      scene->ppc->TranslateDown(tstep);
      scene->Render();
      break;
    }
    case FL_Left: {
      scene->ppc->Pan(rstep);
      scene->Render();
      break;
    }
    case FL_Right: {
      scene->ppc->Pan(-rstep);
      scene->Render();
      break;
    }
    default:
    cerr << "INFO: do not understand keypress" << endl;
  }
}

